﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class Rendevous
    {
        enum TargetType { None, Vessel, Debris, Celestial };    // Type of target.
        FlightEngineer fe;                                      // The calling FlightEngineer object.
        object targetObject;
        Vector2 scrollPosition = Vector2.zero;
        TargetType typeOfTarget = TargetType.None;              // Variable used to hold the type of target.
        bool hasTarget = false;                                 // Variable used to determin if a target is selected.
        bool hasRigidBodyVelocity = false;
        int scrollHeight = 32 * 6;

        // Class constructor that passes in the calling FlightEngineer object.
        public FlightEngineer FlightEngineer
        {
            get { return fe; }
            set { fe = value; }
        }

        public void Draw()
        {
            if (!hasTarget)
            {
                switch (typeOfTarget)
                {
                    case TargetType.Vessel:
                        DrawTargets();
                        break;
                    case TargetType.Celestial:
                        DrawTargets();
                        break;
                    case TargetType.Debris:
                        DrawTargets();
                        break;
                    default:
                        DrawTargetTypes();
                        break;
                }
            }
            else
            {
                DrawDetails();
            }
        }

        private void DrawTargetTypes()
        {
            GUILayout.Label("Rendezvous Display", fe.titleStyle);
            GUILayout.BeginHorizontal(fe.areaStyle);
            GUILayout.BeginVertical();
            if (GUILayout.Button("Vessels", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Vessel;
                fe.winPos.height = 0;
            }
            if (GUILayout.Button("Debris", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Debris;
                fe.winPos.height = 0;
            }
            if (GUILayout.Button("Celestial Bodies", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Celestial;
                fe.winPos.height = 0;
            }
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        private void DrawTargets()
        {
            if (typeOfTarget == TargetType.Celestial)
            {
                GUILayout.Label("Celestial Bodies", fe.titleStyle);
                scrollPosition = GUILayout.BeginScrollView(scrollPosition, fe.scrollStyle, GUILayout.MinHeight(scrollHeight));

                Color buttonTextColor = fe.buttonStyle.normal.textColor;
                fe.buttonStyle.normal.textColor = Color.white;
                foreach (CelestialBody body in fe.activeVessel.orbit.referenceBody.orbitingBodies)
                {
                    if (GUILayout.Button(body.name, fe.buttonStyle))
                    {
                        targetObject = body;
                        hasTarget = true;
                        fe.winPos.height = 0;
                        scrollPosition = Vector2.zero;
                        return;
                    }
                }

                fe.buttonStyle.normal.textColor = Color.cyan;
                foreach (CelestialBody body in Planetarium.fetch.Sun.orbitingBodies)
                {
                    if (GUILayout.Button(body.name, fe.buttonStyle))
                    {
                        targetObject = body;
                        hasTarget = true;
                        fe.winPos.height = 0;
                        scrollPosition = Vector2.zero;
                        return;
                    }
                }

                fe.buttonStyle.normal.textColor = buttonTextColor;

                GUILayout.EndScrollView();
            }
            else if (typeOfTarget != TargetType.None)
            {
                if (typeOfTarget == TargetType.Debris)
                    GUILayout.Label("Debris", fe.titleStyle);

                if (typeOfTarget == TargetType.Vessel)
                    GUILayout.Label("Vessels", fe.titleStyle);

                scrollPosition = GUILayout.BeginScrollView(scrollPosition, fe.scrollStyle, GUILayout.MinHeight(scrollHeight));

                foreach (Vessel v in FlightGlobals.Vessels)
                {
                    if (v == fe.activeVessel) continue;
                    if (v.LandedOrSplashed) continue;

                    if (typeOfTarget == TargetType.Debris)
                        if (v.isCommandable) continue;

                    if (typeOfTarget == TargetType.Vessel)
                        if (!v.isCommandable) continue;

                    if (v.mainBody == fe.activeVessel.mainBody)
                    {
                        if (GUILayout.Button(v.vesselName, fe.buttonStyle))
                        {
                            targetObject = v;
                            hasTarget = true;
                            fe.winPos.height = 0;
                            scrollPosition = Vector2.zero;
                            return;
                        }
                    }
                }

                GUILayout.EndScrollView();
            }

            if (GUILayout.Button("Back to Target Type", fe.buttonStyle))
            {
                targetObject = null;
                hasTarget = false;
                typeOfTarget = TargetType.None;
                fe.winPos.height = 0;
                scrollPosition = Vector2.zero;
            }
        }

        private void DrawDetails()
        {
            if (typeOfTarget == TargetType.Vessel || typeOfTarget == TargetType.Debris)
            {
                Vessel targetVessel = (Vessel)targetObject;
                GUILayout.Label(targetVessel.vesselName, fe.titleStyle);
                DrawVesselDetails(targetVessel);
            }

            if (typeOfTarget == TargetType.Celestial)
            {
                CelestialBody targetBody = (CelestialBody)targetObject;
                GUILayout.Label(targetBody.name, fe.titleStyle);
                DrawBodyDetails(targetBody);
            }

            if (GUILayout.Button("Back to Targets", fe.buttonStyle))
            {
                targetObject = null;
                hasTarget = false;
                fe.winPos.height = 0;
                scrollPosition = Vector2.zero;
            }
        }

        // Used to calculate and draw the rendezvous details if the target is a celestial body.
        private void DrawBodyDetails(CelestialBody targetBody)
        {
            Vector3d activePosition = Vector3d.zero;
            Vector3d targetPosition = Vector3d.zero;
            double phaseAngle = 0.0f;
            double distance = 0.0f;
            double relInclination = 0.0f;
            double altitude = 0.0f;

            targetPosition = targetBody.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());

            if ((fe.activeVessel.mainBody.name == "Sun") || fe.activeVessel.mainBody == targetBody.referenceBody)
            {
                activePosition = fe.activeVessel.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());
            }
            else
            {
                activePosition = fe.activeVessel.mainBody.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());
            }

            phaseAngle = Vector3d.Angle(targetPosition, activePosition);
            if (Vector3d.Angle(Quaternion.AngleAxis(90, Vector3d.forward) * activePosition, targetPosition) > 90) { phaseAngle = 360 - phaseAngle; }
            phaseAngle = (phaseAngle + 360) % 360;

            relInclination = fe.activeVessel.orbit.inclination - targetBody.orbit.inclination;
            altitude = targetBody.orbit.altitude;
            distance = Vector3d.Distance(targetPosition, activePosition);

            GUILayout.BeginHorizontal(fe.areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Phase Angle", fe.headStyle);
            GUILayout.Label("Relative Inclination", fe.headStyle);
            GUILayout.Label("Altitude", fe.headStyle);
            GUILayout.Label("Distance", fe.headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(phaseAngle.ToString("0.000") + "°", fe.dataStyle);
            GUILayout.Label(relInclination.ToString("0.000") + "°", fe.dataStyle);
            GUILayout.Label(fe.FormatSI(altitude, EngineerModule.SIUnitType.Distance), fe.dataStyle);
            GUILayout.Label(fe.FormatSI(distance, EngineerModule.SIUnitType.Distance), fe.dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        // Used to calculate and draw the rendezvous details if the target is a vessel.
        private void DrawVesselDetails(Vessel targetVessel)
        {
            Vector3d activePosition = Vector3d.zero;
            Vector3d targetPosition = Vector3d.zero;
            Vector3d activeVelocity = Vector3d.zero;
            Vector3d targetVelocity = Vector3d.zero;
            Vector3d relVelocity = Vector3d.zero;
            double phaseAngle = 0.0f;
            double distance = 0.0f;
            double relInclination = 0.0f;
            double altitude = 0.0f;
            double velocity = 0.0f;
            double relVelocityMagnitude = 0.0f;

            targetPosition = targetVessel.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());
            activePosition = fe.activeVessel.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());

            phaseAngle = Vector3d.Angle(activePosition, targetPosition);

            relInclination = fe.activeVessel.orbit.inclination - targetVessel.orbit.inclination;
            altitude = targetVessel.orbit.altitude;
            distance = Vector3d.Distance(targetPosition, activePosition);

            activeVelocity = fe.activeVessel.rb_velocity;
            targetVelocity = targetVessel.rb_velocity;
            relVelocity = Vector3d.Project(fe.activeVessel.rb_velocity, targetVessel.rb_velocity.normalized) - targetVessel.rb_velocity;
            if (relVelocity != Vector3d.zero)
            {
                if (!hasRigidBodyVelocity)
                {
                    hasRigidBodyVelocity = true;
                    fe.winPos.height = 0;
                }
            }
            else
            {
                if (hasRigidBodyVelocity)
                {
                    hasRigidBodyVelocity = false;
                    fe.winPos.height = 0;
                }
            }

            activeVelocity = fe.activeVessel.obt_velocity;
            targetVelocity = targetVessel.obt_velocity;
            relVelocityMagnitude = activeVelocity.magnitude - targetVelocity.magnitude;

            velocity = targetVessel.obt_velocity.magnitude;

            GUILayout.BeginHorizontal(fe.areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Phase Angle", fe.headStyle);
            GUILayout.Label("Relative Inclination", fe.headStyle);
            GUILayout.Label("Current Altitude", fe.headStyle);
            GUILayout.Label("Apoapsis Height", fe.headStyle);
            GUILayout.Label("Periapsis Height", fe.headStyle);
            GUILayout.Label("Distance", fe.headStyle);
            GUILayout.Label("Velocity", fe.headStyle);
            GUILayout.Label("Relative Velocity", fe.headStyle);
            if (relVelocity != Vector3d.zero)
            {
                GUILayout.Label("Rel. Velocity X", fe.headStyle);
                GUILayout.Label("Rel. Velocity Y", fe.headStyle);
                GUILayout.Label("Rel. Velocity Z", fe.headStyle);
            }
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(phaseAngle.ToString("0.000") + "°", fe.dataStyle);
            GUILayout.Label(relInclination.ToString("0.000") + "°", fe.dataStyle);
            GUILayout.Label(fe.FormatSI(altitude, EngineerModule.SIUnitType.Distance), fe.dataStyle);
            GUILayout.Label(fe.FormatSI(targetVessel.orbit.ApA, EngineerModule.SIUnitType.Distance), fe.dataStyle);
            GUILayout.Label(fe.FormatSI(targetVessel.orbit.PeA, EngineerModule.SIUnitType.Distance), fe.dataStyle);
            GUILayout.Label(fe.FormatSI(distance, EngineerModule.SIUnitType.Distance), fe.dataStyle);
            GUILayout.Label(fe.FormatSI(velocity, EngineerModule.SIUnitType.Speed), fe.dataStyle);
            GUILayout.Label(fe.FormatSI(relVelocityMagnitude, EngineerModule.SIUnitType.Speed), fe.dataStyle);
            if (relVelocity != Vector3d.zero)
            {
                GUILayout.Label(fe.FormatSI(relVelocity.x, EngineerModule.SIUnitType.Speed), fe.dataStyle);
                GUILayout.Label(fe.FormatSI(relVelocity.y, EngineerModule.SIUnitType.Speed), fe.dataStyle);
                GUILayout.Label(fe.FormatSI(relVelocity.z, EngineerModule.SIUnitType.Speed), fe.dataStyle);
            }
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        public CelestialBody GetBodyByName(string name)
        {
            foreach (CelestialBody body in FlightGlobals.Bodies)
            {
                if (body.name == name)
                {
                    return body;
                }
            }

            return null;
        }
    }
}


//Vector3d active = Vector3d.zero;
//Vector3d target = Vector3d.zero;
//string name = null;
//double phaseAngle = 0.0f;
//double distance = 0.0f;
//double relInclination = 0.0f;
//double altitude = 0.0f;
//double relVelocity = 0.0f;
//double gravity = 0.0f;

//if (typeOfTarget == TargetType.Vessel || typeOfTarget == TargetType.Debris)
//{
//    name = targetVes.vesselName;
//    phaseAngle = Vector3d.Angle(fe.activeVessel.orbit.referenceBody.position - fe.activeVessel.CoM, targetVes.orbit.referenceBody.position - targetVes.CoM);
//    relInclination = targetVes.orbit.inclination - fe.activeVessel.orbit.inclination;
//    altitude = targetVes.orbit.altitude;
//    relVelocity = targetVes.orbit.vel.magnitude - fe.activeVessel.orbit.vel.magnitude;
//    distance = Vector3d.Distance(fe.activeVessel.CoM, targetVes.CoM);
//}
//else if (typeOfTarget == TargetType.Celestial)
//{
//    name = targetBody.name;
//    Vector3d vectorVessel = Vector3d.zero;
//    Vector3d vectorTarget = targetBody.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());

//    if ((fe.activeVessel.mainBody.name == "Sun") || fe.activeVessel.mainBody == targetBody.referenceBody)
//    {
//        vectorVessel = fe.activeVessel.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime());
//    }
//    else
//    {
//        vectorVessel = fe.activeVessel.mainBody.orbit.getRelativePositionAtUT(Planetarium.GetUniversalTime()); 
//    }

//    phaseAngle = Vector3d.Angle(vectorVessel, vectorTarget);
//    if (Vector3d.Angle(Quaternion.AngleAxis(90, Vector3d.forward) * vectorVessel, vectorTarget) > 90) { phaseAngle = 360 - phaseAngle; }
//    phaseAngle = (phaseAngle + 360) % 360;
//    relInclination = targetBody.orbit.inclination - fe.activeVessel.orbit.inclination;
//    altitude = targetBody.orbit.altitude;
//    distance = Vector3d.Distance(fe.activeVessel.CoM, targetBody.position);
//    gravity = targetBody.gMagnitudeAtCenter;
//}

//GUILayout.Label(name, fe.titleStyle);

//GUILayout.BeginHorizontal(fe.areaStyle);
//GUILayout.BeginVertical();
//GUILayout.Label("Phase Angle", fe.headStyle);
//GUILayout.Label("Relative Inclination", fe.headStyle);
//GUILayout.Label("Altitude", fe.headStyle);
//GUILayout.Label("Distance", fe.headStyle);
//if (typeOfTarget != TargetType.Celestial) GUILayout.Label("Relative Velocity", fe.headStyle);
//GUILayout.EndVertical();

//GUILayout.BeginVertical();
//GUILayout.Label(phaseAngle.ToString("0.000") + "°", fe.dataStyle);
//GUILayout.Label(relInclination.ToString("0.000") + "°", fe.dataStyle);
//GUILayout.Label(fe.FormatDistance(altitude), fe.dataStyle);
//GUILayout.Label(fe.FormatDistance(distance), fe.dataStyle);
//if (typeOfTarget != TargetType.Celestial) GUILayout.Label(fe.FormatSpeed(relVelocity), fe.dataStyle);
//GUILayout.EndVertical();
//GUILayout.EndHorizontal();