﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class Stage
    {
        public Ship Ship { get; private set; }
        public List<Part> Parts { get; private set; }
        public int StartPartCount { get; private set; }
        public int StageNumber { get; private set; }
        public float Mass { get; private set; }
        public int Cost { get; private set; }
        public int TotalCost { get; private set; }
        public float TotalMass { get; private set; }
        public float DryMass { get; private set; }
        public float FuelMass { get; private set; }
        public float Fuel { get; private set; }
        public float TotalFuel { get; private set; }
        public float Isp { get; private set; }
        public float AtmIsp { get; private set; }
        public float VacIsp { get; private set; }
        public float Thrust { get; private set; }
        public float DeltaV { get; private set; }
        public float TotalDeltaV { get; private set; }

        public bool Changed
        {
            get
            {
                if (Parts.Count != StartPartCount)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        public bool Exists
        {
            get
            {
                foreach (Part part in Parts)
                {
                    if (part.isAttached && part.inverseStage == StageNumber)
                        return true;
                }
                return false;
            }
        }

        public bool Important
        {
            get
            {
                if (DeltaV > 0)
                    return true;
                return false;
            }
        }

        public Stage(int stageNumber, Ship ship)
        {
            StageNumber = stageNumber;
            Ship = ship;
            Parts = new List<Part>();
            StartPartCount = Parts.Count;
        }

        public bool Update(List<Part> parts)
        {
            if (!GetParts(parts))
            {
                foreach (Part part in parts)
                {
                    if (part.inverseStage > StageNumber)
                    {
                        GetMass();
                        return true;
                    }
                }
                return false;
            }

            GetCost();
            GetMass();
            GetFuel();
            GetIsp();
            GetThrust();
            GetDeltaV();

            return true;
        }

        private bool GetParts(List<Part> parts)
        {
            Parts.Clear();

            foreach (Part part in parts)
            {
                if (part.inverseStage == this.StageNumber)
                    Parts.Add(part);
            }

            if (Parts.Count == 0)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        private void GetCost()
        {
            int cost = 0;

            foreach (Part part in Parts)
            {
                cost += part.partInfo.cost;
            }

            Cost = cost;

            int totalCost = 0;

            for (int i = 0; i < StageNumber; i++)
            {
                totalCost += Ship.Stages[i].Cost;
            }

            totalCost += cost;
            TotalCost = totalCost;
        }

        private void GetMass()
        {
            float mass = 0.0f;
            float dryMass = 0.0f;

            foreach (Part part in Parts)
            {
                mass += part.mass;
                dryMass += part.mass;
                if (part is FuelTank)
                {
                    dryMass += (part as FuelTank).dryMass;
                    dryMass -= part.mass;
                }
                if (part is SolidRocket)
                {
                    dryMass += (part as SolidRocket).dryMass;
                    dryMass -= part.mass;

                }
            }

            Mass = mass;
            DryMass = dryMass;
            FuelMass = mass - dryMass;

            float totalMass = 0.0f;

            for (int i = 0; i < StageNumber; i++)
            {
                totalMass += Ship.Stages[i].Mass;
            }
            totalMass += mass;
            TotalMass = totalMass;
        }

        private void GetFuel()
        {
            float fuel = 0.0f;

            foreach (Part part in Parts)
            {
                if (part is FuelTank)
                    fuel += (part as FuelTank).fuel;
            }

            Fuel = fuel;

            float totalFuel = 0.0f;

            for (int i = 0; i < StageNumber; i++)
            {
                totalFuel += Ship.Stages[i].Fuel;
            }

            totalFuel += fuel;
            TotalFuel = totalFuel;
        }

        private void GetIsp()
        {
            float isp = 0.0f;
            float thrust = 0.0f;

            int aIspCount = 0;
            float aIsp = 0.0f;
            int vIspCount = 0;
            float vIsp = 0.0f;

            foreach (Part part in Parts)
            {
                if (part is SolidRocket)
                {
                    float thisIsp = 0.0f;
                    thisIsp = (part as SolidRocket).thrust / ((((part as SolidRocket).mass - (part as SolidRocket).dryMass) / ((part as SolidRocket).internalFuel / (part as SolidRocket).fuelConsumption)) * 10);
                    isp += thisIsp * (part as SolidRocket).thrust;
                    thrust += (part as SolidRocket).thrust;
                    aIsp += thisIsp;
                    aIspCount++;
                    vIsp += thisIsp;
                    vIspCount++;
                }
                if (part is LiquidEngine)
                {
                    float thisIsp = 0.0f;
                    thisIsp = (((part as LiquidEngine).maxThrust * (part as LiquidEngine).fuelConsumption) * 0.001f) * (part as LiquidEngine).maxThrust;
                    isp += thisIsp * (part as LiquidEngine).maxThrust;
                    thrust += (part as LiquidEngine).maxThrust;
                    aIsp += thisIsp;
                    aIspCount++;
                    vIsp += thisIsp;
                    vIspCount++;
                }
                if (part is LiquidFuelEngine || part.ClassName == "LiquidFuelEngineFix")
                {
                    if (Ship.Engineer.state != PartModule.StartState.Editor)
                    {
                        isp += (part as LiquidFuelEngine).realIsp * (part as LiquidFuelEngine).maxThrust;
                    }
                    else
                    {
                        isp += (part as LiquidFuelEngine).vacIsp * (part as LiquidFuelEngine).maxThrust;
                    }
                    thrust += (part as LiquidFuelEngine).maxThrust;
                    aIsp += (part as LiquidFuelEngine).Isp;
                    aIspCount++;
                    vIsp += (part as LiquidFuelEngine).vacIsp;
                    vIspCount++;
                }
                if (part is AtmosphericEngine)
                {
                    isp += (part as AtmosphericEngine).maximumExhaustSpeed * (part as AtmosphericEngine).maximumEnginePower;
                    thrust += (part as AtmosphericEngine).maximumEnginePower;
                    aIsp += (part as AtmosphericEngine).maximumExhaustSpeed;
                    aIspCount++;
                }
            }

            if (isp > 0 && thrust > 0)
            {
                Isp = isp / thrust;
            }
            else
            {
                Isp = 0;
            }

            if (aIsp > 0 && aIspCount > 0)
            {
                AtmIsp = aIsp / aIspCount;
            }
            else
            {
                AtmIsp = 0;
            }

            if (vIsp > 0 && vIspCount > 0)
            {
                VacIsp = vIsp / vIspCount;
            }
            else
            {
                VacIsp = 0;
            }
        }

        private void GetThrust()
        {
            float thrust = 0.0f;

            foreach (Part part in Parts)
            {
                if (part is LiquidEngine)
                    thrust += (part as LiquidEngine).maxThrust;
                if (part is LiquidFuelEngine)
                    thrust += (part as LiquidFuelEngine).maxThrust;
                if (part is SolidRocket)
                    thrust += (part as SolidRocket).thrust;
                if (part is AtmosphericEngine)
                    thrust += (part as AtmosphericEngine).maximumEnginePower;
            }

            Thrust = thrust;
        }

        private void GetDeltaV()
        {
            DeltaV = -(float)((Isp * 9.81) * Mathf.Log((TotalMass - FuelMass) / TotalMass));

            float totalDeltaV = 0.0f;
            for (int i = 0; i < StageNumber; i++)
            {
                totalDeltaV += Ship.Stages[i].DeltaV;
            }

            totalDeltaV += DeltaV;
            TotalDeltaV = totalDeltaV;
        }
    }
}
